/*
 Application.cc - app launcher

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall
*/

#include "Application.h"
#include "MainWindow.h"

#include <assert.h>
#include <stdlib.h>


Application *application = NULL;


Application::Application ( const char *appClassName ) : Interface ( appClassName )
{	// set global application pointer and initialize data members
	application = this;	
	_display = NULL;
	_appContext = NULL;
	_applicationClass = XtNewString ( appClassName );
}

/* Note: 'XtOpenApplication' formerly 'XtAppInitialize',
  'sessionShellWidgetClass' formerly 'applicationShellWidgetClass'.
  The new terms apply from at least XFree86-4.1.0, possibly earlier.
*/

void Application::initialize ( int *argc, char *argv[] )
{
	_w = XtOpenApplication ( &_appContext, _applicationClass, NULL, 0,
							 argc, argv, fallback_resources,
							 sessionShellWidgetClass, NULL, 0 );

	_display = XtDisplay ( _w );	// store pointer to X display structure
	installDestroyHandler();	// in case of 'surprise' widget destruction

	XtVaSetValues ( _w, XmNmappedWhenManaged, FALSE,
			XmNx, DisplayWidth ( _display, 0 ) / 2,
			XmNy, DisplayHeight ( _display, 0 ) / 2,
			XmNwidth, 1, XmNheight, 1, NULL );
	
	XtFree ( _name );				 // free instance name set in Interface constructor
	_name = XtNewString ( argv[0] ); // program name unavailable at the time
	XtRealizeWidget ( _w );			 // realise shell window

	// initialise and manage windows registered with this application
	for ( int i = 0; i < _windows.Count(); i++ )
	{
		_windows[i]->initialize();
		_windows[i]->manage();
	}
}


Application::~Application()
{
	XtFree ( _applicationClass );
}


void Application::handleEvents()
{
	XtAppMainLoop ( _appContext );	// go into Xt loop
}


void Application::registerWindow ( MainWindow *window )
{
	_windows.Add ( window );
}


void Application::unregisterWindow ( MainWindow *window )
{
	_windows.Remove ( window );
} 


void Application::manage()
{
		for ( int i = 0; i < _windows.Count(); i++ )
			_windows[i]->manage();
}


void Application::unmanage()
{
	for ( int i = 0; i < _windows.Count(); i++ )
		_windows[i]->unmanage();
}


void Application::iconify()
{
	for ( int i = 0; i < _windows.Count(); i++ )
		_windows[i]->iconify();
}


/*
  Notes:
  
  Application class constructor doesn't create any widgets; accepts an argument
  containing the class name of the application; passes this to Interface as the
  name of the instance. This is later substituted by argv[0], the program name.
  
  The shell widget returned by XtOpenApplication is the application's base
  widget.
  
  The Interface destruction handler caters for widgets destroyed before the class
  destructor is called. Applications could destroy the base widget by calling
  XtDestroyWidget() on the value returned by baseWidget(). To deal with this and
  mainatin consistency, installDestroyHandler() is called.
  
  This app framework supports apps with multiple top-level windows by creating a
  single widget as the parent of all top-level windows. This base widget is not
  realised.
  
  The XtVaSetValues resource specification supports classes caching dialogs. The
  Application class' base widget is used as the parent of all cached dialogs.
  Each dialog is centred over its parent widget (if the latter has a window).
  
  The Application destructor frees the string allocated in either the
  constructor or in the initialize() member function. The Foundation destructor
  destroys both the the string assigned to _name and the base widget.
  
  Top-level windows are implemented as instances of the MainWindow class. The
  Application member functions registerWindow() and unregisterWindow() allow each
  instance of MainWindow to register itself with the Application class on
  creation and to unregister itself on destruction.

  Derived classes can support additional operations  in manage(), unmanage() and
  iconify(). The manage() and unmanage() member functions override those in
  Interface.
*/
